@@ -9,38 +9,33 @@
 
 namespace HandBrakeWPF.ViewModels
 {
+    using System;
     using System.Collections.Generic;
     using System.Collections.ObjectModel;
-    using System.ComponentModel;
     using System.Linq;
 
-    using Caliburn.Micro;
-
-    using HandBrake.ApplicationServices.Interop;
-    using HandBrake.ApplicationServices.Interop.Model.Encoding;
-    using HandBrake.ApplicationServices.Utilities;
+    using HandBrake.Interop.Interop;
+    using HandBrake.Interop.Interop.Interfaces.Model.Encoders;
+    using HandBrake.Interop.Utilities;
 
+    using HandBrakeWPF.Commands;
+    using HandBrakeWPF.EventArgs;
     using HandBrakeWPF.Model.Audio;
-    using HandBrakeWPF.Properties;
     using HandBrakeWPF.Services.Interfaces;
     using HandBrakeWPF.Services.Presets.Model;
     using HandBrakeWPF.Services.Scan.Model;
-    using HandBrakeWPF.Utilities;
     using HandBrakeWPF.ViewModels.Interfaces;
 
-    using AudioEncoder = HandBrakeWPF.Services.Encode.Model.Models.AudioEncoder;
-    using AudioTrack = HandBrakeWPF.Services.Encode.Model.Models.AudioTrack;
-    using EncodeTask = HandBrakeWPF.Services.Encode.Model.EncodeTask;
-    using OutputFormat = HandBrakeWPF.Services.Encode.Model.Models.OutputFormat;
+    using AudioTrack = Services.Encode.Model.Models.AudioTrack;
+    using EncodeTask = Services.Encode.Model.EncodeTask;
+    using OutputFormat = Services.Encode.Model.Models.OutputFormat;
 
     /// <summary>
     /// The Audio View Model
     /// </summary>
     public class AudioViewModel : ViewModelBase, IAudioViewModel
     {
-        private readonly IWindowManager windowManager;
         private IEnumerable<Audio> sourceTracks;
-        private Preset currentPreset;
 
         #region Constructors and Destructors
 
@@ -55,9 +50,8 @@ namespace HandBrakeWPF.ViewModels
         /// </param>
         public AudioViewModel(IWindowManager windowManager, IUserSettingService userSettingService)
         {
-            this.windowManager = windowManager;
             this.Task = new EncodeTask();
-            this.AudioDefaultsViewModel = new AudioDefaultsViewModel(this.Task);
+            this.AudioDefaultsViewModel = new AudioDefaultsViewModel(windowManager);
 
             this.SampleRates = new ObservableCollection<string> { "Auto" };
             foreach (var item in HandBrakeEncoderHelpers.AudioSampleRates)
@@ -65,12 +59,17 @@ namespace HandBrakeWPF.ViewModels
                 this.SampleRates.Add(item.Name);
             }
 
-            this.AudioEncoders = EnumHelper<AudioEncoder>.GetEnumList();
+            this.AudioEncoders = HandBrakeEncoderHelpers.AudioEncoders.ToList();
             this.SourceTracks = new List<Audio>();
+            this.RemoveCommand = new SimpleRelayCommand<AudioTrack>(this.Remove);
         }
 
+        public SimpleRelayCommand<AudioTrack> RemoveCommand { get; set; }
+
         #endregion
 
+        public event EventHandler<TabStatusEventArgs> TabStatusChanged;
+
         #region Properties
 
         /// <summary>
@@ -78,10 +77,12 @@ namespace HandBrakeWPF.ViewModels
         /// </summary>
         public IAudioDefaultsViewModel AudioDefaultsViewModel { get; set; }
 
+        public ListboxDeleteCommand DeleteCommand => new ListboxDeleteCommand();
+
         /// <summary>
         /// Gets or sets AudioEncoders.
         /// </summary>
-        public IEnumerable<AudioEncoder> AudioEncoders { get; set; }
+        public IEnumerable<HBAudioEncoder> AudioEncoders { get; set; }
 
         /// <summary>
         /// Gets or sets SampleRates.
@@ -109,38 +110,10 @@ namespace HandBrakeWPF.ViewModels
         /// </summary>
         public EncodeTask Task { get; set; }
 
-        /// <summary>
-        /// Gets the panel title.
-        /// </summary>
-        public string PanelTitle
-        {
-            get
-            {
-                return Resources.AudioViewModel_AudioTracks;
-            }
-        }
-
-        /// <summary>
-        /// Gets the switch display title.
-        /// </summary>
-        public string SwitchDisplayTitle
-        {
-            get
-            {
-                return Resources.AudioViewModel_ConfigureDefaults;
-            }
-        }
-
         /// <summary>
         /// Gets the default audio behaviours. 
         /// </summary>
-        public AudioBehaviours AudioBehaviours
-        {
-            get
-            {
-                return this.AudioDefaultsViewModel.AudioBehaviours;
-            }
-        }
+        public AudioBehaviours AudioBehaviours { get; private set; }
 
         #endregion
 
@@ -193,19 +166,33 @@ namespace HandBrakeWPF.ViewModels
         /// <summary>
         /// Trigger a Notify Property Changed on the Task to force various UI elements to update.
         /// </summary>
-        public void RefreshTask()
+        public void RefreshTask(OutputFormat format)
         {
-            this.NotifyOfPropertyChange(() => this.Task);
-
-            if (this.Task.OutputFormat == OutputFormat.Mp4)
+            if (format == OutputFormat.Mp4)
             {
-                foreach (AudioTrack track in this.Task.AudioTracks.Where(track => track.Encoder == AudioEncoder.ffflac || track.Encoder == AudioEncoder.Vorbis))
+                foreach (AudioTrack track in this.Task.AudioTracks)
                 {
-                    track.Encoder = AudioEncoder.ffaac;
+                    if (!track.Encoder.SupportsMP4)
+                    {
+                        track.Encoder = HandBrakeEncoderHelpers.GetAudioEncoder(HBAudioEncoder.AvAac);
+                    }
                 }
             }
 
-            this.AudioDefaultsViewModel.RefreshTask();
+            if (format == OutputFormat.WebM)
+            {
+                foreach (AudioTrack track in this.Task.AudioTracks)
+                {
+                    if (!track.Encoder.SupportsWebM)
+                    {
+                        track.Encoder = HandBrakeEncoderHelpers.GetAudioEncoder(HBAudioEncoder.Vorbis);
+                    }
+                }
+            }
+
+            this.NotifyOfPropertyChange(() => this.Task); // Trigger UI Refresh
+
+            this.AudioDefaultsViewModel.RefreshTask(format);
         }
 
         /// <summary>
@@ -213,14 +200,30 @@ namespace HandBrakeWPF.ViewModels
         /// </summary>
         public void ShowAudioDefaults()
         {
-            IPopupWindowViewModel popup = new PopupWindowViewModel(this.AudioDefaultsViewModel, ResourcesUI.Preset_AudioDefaults_Title, ResourcesUI.AudioView_AudioDefaultsDescription);
-            if (this.windowManager.ShowDialog(popup) == true)
+            this.AudioDefaultsViewModel.Setup(new AudioBehaviours(this.AudioBehaviours), this.Task.OutputFormat);
+
+            if (this.AudioDefaultsViewModel.ShowWindow())
             {
-                // Nothing to do yet, it's by reference. 
+                this.AudioBehaviours = new AudioBehaviours(this.AudioDefaultsViewModel.AudioBehaviours);
+                this.Task.AudioPassthruOptions = this.AudioBehaviours.AllowedPassthruOptions;
+
+                this.OnTabStatusChanged(null);
             }
-            else
+        }
+
+        public void ExpandAllTracks()
+        {
+            foreach (var track in this.Task.AudioTracks)
             {
-                // Handle other case(s)
+                track.IsExpandedTrackView = true;
+            }
+        }
+
+        public void CollapseAllTracks()
+        {
+            foreach (var track in this.Task.AudioTracks)
+            {
+                track.IsExpandedTrackView = false;
             }
         }
 
@@ -240,12 +243,13 @@ namespace HandBrakeWPF.ViewModels
         public void SetPreset(Preset preset, EncodeTask task)
         {
             this.Task = task;
-            this.currentPreset = preset;
 
             // Audio Behaviours
-            this.AudioDefaultsViewModel.Setup(preset, task);
-
-            if (preset != null && preset.Task != null)
+            this.AudioDefaultsViewModel.Setup(preset.AudioTrackBehaviours, preset.Task.OutputFormat);
+            this.AudioBehaviours = new AudioBehaviours(preset.AudioTrackBehaviours);
+            this.Task.AudioPassthruOptions = this.AudioBehaviours.AllowedPassthruOptions;
+            
+            if (preset.Task != null)
             {
                 this.SetupTracks();
             }
@@ -266,6 +270,72 @@ namespace HandBrakeWPF.ViewModels
             this.NotifyOfPropertyChange(() => this.Task);
         }
 
+        public bool MatchesPreset(Preset preset)
+        {
+            // Check the default behaviours still match the preset.
+            if (preset.AudioTrackBehaviours.SelectedBehaviour != this.AudioBehaviours.SelectedBehaviour)
+            {
+                return false;
+            }
+
+            if (preset.AudioTrackBehaviours.SelectedTrackDefaultBehaviour
+                != this.AudioBehaviours.SelectedTrackDefaultBehaviour)
+            {
+                return false;
+            }
+
+            foreach (var item in this.AudioBehaviours.SelectedLanguages)
+            {
+                if (!preset.AudioTrackBehaviours.SelectedLanguages.Contains(item))
+                {
+                    return false;
+                }
+            }
+
+            // Check if we have missing fallback options.
+            foreach (HBAudioEncoder encoder in this.Task.AudioPassthruOptions)
+            {
+                if (!preset.AudioTrackBehaviours.AllowedPassthruOptions.Contains(encoder))
+                {
+                    return false;
+                }
+            }
+
+            // Other direction
+            foreach (HBAudioEncoder encoder in preset.AudioTrackBehaviours.AllowedPassthruOptions)
+            {
+                if (!this.Task.AudioPassthruOptions.Contains(encoder))
+                {
+                    return false;
+                }
+            }
+
+            if (preset.AudioTrackBehaviours.AudioFallbackEncoder != this.Task.AudioFallbackEncoder)
+            {
+                return false;
+            }
+
+            foreach (var language in preset.AudioTrackBehaviours.SelectedLanguages)
+            {
+                if (!this.AudioBehaviours.SelectedLanguages.Contains(language))
+                {
+                    return false;
+                }
+            }
+
+            if (preset.AudioTrackBehaviours.SelectedLanguages.Count != this.AudioBehaviours.SelectedLanguages.Count)
+            {
+                return false;
+            }
+
+            if (preset.AudioTrackBehaviours.BehaviourTracks.Count != this.AudioBehaviours.BehaviourTracks.Count)
+            {
+                return false;
+            }
+
+            return true;
+        }
+
         /// <summary>
         /// Set the Source Title
         /// </summary>
@@ -308,6 +378,10 @@ namespace HandBrakeWPF.ViewModels
         #endregion
 
         #region Methods
+        protected virtual void OnTabStatusChanged(TabStatusEventArgs e)
+        {
+            this.TabStatusChanged?.Invoke(this, e);
+        }
 
         /// <summary>
         /// Add the specified source track, or the first track in the SourceTracks collection if available.
@@ -333,17 +407,20 @@ namespace HandBrakeWPF.ViewModels
 
                     switch (this.AudioBehaviours.SelectedTrackDefaultBehaviour)
                     {
-                        case AudioTrackDefaultsMode.None:
-                            this.Task.AudioTracks.Add(new AudioTrack { ScannedTrack = track });
-                            break;
                         case AudioTrackDefaultsMode.FirstTrack:
                             AudioBehaviourTrack template = this.AudioBehaviours.BehaviourTracks.FirstOrDefault();
-                            this.Task.AudioTracks.Add(template != null ? new AudioTrack(template) { ScannedTrack = track } : new AudioTrack { ScannedTrack = track });
+                            if (this.CanAddTrack(template, track, this.AudioBehaviours.AudioFallbackEncoder))
+                            {
+                                this.Task.AudioTracks.Add( template != null ? new AudioTrack(template, track, this.AudioBehaviours.AllowedPassthruOptions, this.AudioBehaviours.AudioFallbackEncoder, this.Task.OutputFormat) : new AudioTrack { ScannedTrack = track });
+                            }
                             break;
                         case AudioTrackDefaultsMode.AllTracks:
                             foreach (AudioBehaviourTrack tmpl in this.AudioBehaviours.BehaviourTracks)
                             {
-                                this.Task.AudioTracks.Add(tmpl != null ? new AudioTrack(tmpl) { ScannedTrack = track } : new AudioTrack { ScannedTrack = track });
+                                if (this.CanAddTrack(tmpl, track, this.AudioBehaviours.AudioFallbackEncoder))
+                                {
+                                    this.Task.AudioTracks.Add(tmpl != null ? new AudioTrack(tmpl, track, this.AudioBehaviours.AllowedPassthruOptions, this.AudioBehaviours.AudioFallbackEncoder, this.Task.OutputFormat) : new AudioTrack { ScannedTrack = track });
+                                }
                             }
 
                             break;
@@ -352,22 +429,17 @@ namespace HandBrakeWPF.ViewModels
             }
         }
 
-        /// <summary>
-        /// Add all source tracks that don't currently exist on the list.
-        /// </summary>
-        private void AddAllRemainingTracks()
+        private bool CanAddTrack(AudioBehaviourTrack track, Audio sourceTrack, HBAudioEncoder fallback)
         {
-            // For all the source audio tracks
-            foreach (Audio sourceTrack in this.SourceTracks)
+            if (fallback == HBAudioEncoder.None && track != null)
             {
-                // Step 2: Check if the track list already contrains this track
-                bool found = this.Task.AudioTracks.Any(audioTrack => Equals(audioTrack.ScannedTrack, sourceTrack));
-                if (!found)
+                if (track.IsPassthru && (sourceTrack.Codec & track.Encoder.Id) == 0)
                 {
-                    // If it doesn't, add it.
-                    this.Add(sourceTrack, true);
+                    return false;
                 }
             }
+
+            return true;
         }
 
         /// <summary>
@@ -395,7 +467,11 @@ namespace HandBrakeWPF.ViewModels
             // Step 3, Setup the tracks from the preset
             foreach (AudioBehaviourTrack track in this.AudioBehaviours.BehaviourTracks)
             {
-                this.Task.AudioTracks.Add(new AudioTrack(track) { ScannedTrack = this.GetPreferredAudioTrack() });
+                Audio sourceTrack = this.GetPreferredAudioTrack();
+                if (this.CanAddTrack(track, sourceTrack, this.AudioBehaviours.AudioFallbackEncoder))
+                {
+                    this.Task.AudioTracks.Add(new AudioTrack(track, sourceTrack, this.AudioBehaviours.AllowedPassthruOptions, this.AudioBehaviours.AudioFallbackEncoder, this.Task.OutputFormat));
+                }
             }
            
             // Step 4, Handle the default selection behaviour.
@@ -407,7 +483,7 @@ namespace HandBrakeWPF.ViewModels
                 case AudioBehaviourModes.FirstMatch: // Adding all remaining audio tracks
                     this.AddFirstForSelectedLanguages();
                     break;
-                case AudioBehaviourModes.AllMatching: // Add Langauges tracks for the additional languages selected, in-order.
+                case AudioBehaviourModes.AllMatching: // Add Languages tracks for the additional languages selected, in-order.
                     this.AddAllRemainingForSelectedLanguages();
                     break;
             }
@@ -418,9 +494,16 @@ namespace HandBrakeWPF.ViewModels
         /// </summary>
         private void AddFirstForSelectedLanguages()
         {
+            bool anyLanguageSelected = this.AudioBehaviours.SelectedLanguages.Contains(Constants.Any);
+
+            if (anyLanguageSelected && this.Task.AudioTracks.Count >= 1)
+            {
+                return;
+            }
+
             foreach (Audio sourceTrack in this.GetSelectedLanguagesTracks())
             {
-                // Step 2: Check if the track list already contrains this track
+                // Step 2: Check if the track list already contains this track
                 bool found = this.Task.AudioTracks.Any(audioTrack => Equals(audioTrack.ScannedTrack, sourceTrack));
                 if (!found)
                 {
@@ -438,6 +521,30 @@ namespace HandBrakeWPF.ViewModels
 
                     // If it doesn't, add it.
                     this.Add(sourceTrack, true);
+
+                    // If we are using "(Any)" then break here. We only add the first track in this instance.
+                    if (anyLanguageSelected)
+                    {
+                        break;
+                    }
+                }
+            }
+        }
+
+        /// <summary>
+        /// Add all source tracks that don't currently exist on the list.
+        /// </summary>
+        private void AddAllRemainingTracks()
+        {
+            // For all the source audio tracks
+            foreach (Audio sourceTrack in this.SourceTracks)
+            {
+                // Step 2: Check if the track list already contains this track
+                bool found = this.Task.AudioTracks.Any(audioTrack => Equals(audioTrack.ScannedTrack, sourceTrack));
+                if (!found)
+                {
+                    // If it doesn't, add it.
+                    this.Add(sourceTrack, true);
                 }
             }
         }
@@ -450,7 +557,7 @@ namespace HandBrakeWPF.ViewModels
             // Add them if they are not already added.
             foreach (Audio sourceTrack in this.GetSelectedLanguagesTracks())
             {
-                // Step 2: Check if the track list already contrains this track
+                // Step 2: Check if the track list already contains this track
                 bool found = this.Task.AudioTracks.Any(audioTrack => Equals(audioTrack.ScannedTrack, sourceTrack));
                 if (!found)
                 {
@@ -471,12 +578,13 @@ namespace HandBrakeWPF.ViewModels
             // The first track in the selected languages list is considered the preferred language.
             // So, try match tracks on this.
             IEnumerable<Audio> preferredAudioTracks = new List<Audio>();
-            if (this.AudioBehaviours.SelectedLangauges.Count > 0)
+            if (this.AudioBehaviours.SelectedLanguages.Count > 0)
             {
-                string langName = this.AudioBehaviours.SelectedLangauges.FirstOrDefault(w => !w.Equals(Constants.Any));
-                if (!string.IsNullOrEmpty(langName))
+                string langName = this.AudioBehaviours.SelectedLanguages.FirstOrDefault(w => !w.Equals(Constants.Any));
+                string langCode = LanguageUtilities.GetLanguageCode(langName);
+                if (!string.IsNullOrEmpty(langCode))
                 {
-                    preferredAudioTracks = this.SourceTracks.Where(item => item.Language.Contains(langName));
+                    preferredAudioTracks = this.SourceTracks.Where(item => item.LanguageCode.Contains(langCode));
                 }
             }
 
@@ -491,21 +599,25 @@ namespace HandBrakeWPF.ViewModels
         /// </returns>
         private IEnumerable<Audio> GetSelectedLanguagesTracks()
         {
-            List<Audio> trackList = new List<Audio>();
-
-            List<string> isoCodes = LanguageUtilities.GetLanguageCodes(this.AudioBehaviours.SelectedLangauges.ToArray());
-
-            if (isoCodes.Contains(Constants.Undefined))
+            // Translate to Iso Codes
+            List<string> iso6392Codes = new List<string>();
+            if (this.AudioBehaviours.SelectedLanguages.Contains(Constants.Any))
             {
-                isoCodes = LanguageUtilities.GetIsoCodes();
+                iso6392Codes = LanguageUtilities.GetIsoCodes();
+                iso6392Codes = LanguageUtilities.OrderIsoCodes(iso6392Codes, this.AudioBehaviours.SelectedLanguages);
+            }
+            else
+            {
+                iso6392Codes = LanguageUtilities.GetLanguageCodes(this.AudioBehaviours.SelectedLanguages.ToArray());
+            }
+            
+            List<Audio> orderedTracks = new List<Audio>();
+            foreach (string code in iso6392Codes)
+            {
+                orderedTracks.AddRange(this.SourceTracks.Where(audio => audio.LanguageCode == code));
             }
 
-            foreach (string code in isoCodes)
-            {
-                trackList.AddRange(this.SourceTracks.Where(source => source.LanguageCode.Trim() == code));
-            }
-
-            return trackList;
+            return orderedTracks;
         }
 
         #endregion
