@@ -12,67 +12,41 @@ namespace HandBrakeWPF.ViewModels
     using System;
     using System.Collections.Generic;
     using System.Collections.ObjectModel;
-    using System.ComponentModel.DataAnnotations;
     using System.IO;
-    using System.Text;
     using System.Linq;
     using System.Windows;
-    using System.Windows.Forms;
 
-    using Caliburn.Micro;
+    using HandBrake.App.Core.Utilities;
 
+    using HandBrakeWPF.EventArgs;
     using HandBrakeWPF.Properties;
     using HandBrakeWPF.Services.Interfaces;
     using HandBrakeWPF.Services.Presets.Model;
     using HandBrakeWPF.Services.Scan.Model;
     using HandBrakeWPF.Utilities.Input;
-    using HandBrakeWPF.Utilities.Output;
     using HandBrakeWPF.ViewModels.Interfaces;
 
-    using ChapterMarker = HandBrakeWPF.Services.Encode.Model.Models.ChapterMarker;
-    using EncodeTask = HandBrakeWPF.Services.Encode.Model.EncodeTask;
-    using GeneralApplicationException = HandBrakeWPF.Exceptions.GeneralApplicationException;
-    using MessageBox = System.Windows.Forms.MessageBox;
+    using Microsoft.Win32;
+
+    using ChapterMarker = Services.Encode.Model.Models.ChapterMarker;
+    using EncodeTask = Services.Encode.Model.EncodeTask;
+    using GeneralApplicationException = HandBrake.App.Core.Exceptions.GeneralApplicationException;
 
-    /// <summary>
-    /// The Chapters View Model
-    /// </summary>
     public class ChaptersViewModel : ViewModelBase, IChaptersViewModel
     {
-        #region Constants and Fields
-
         private readonly IErrorService errorService;
-
-        /// <summary>
-        /// The source chapters backing field
-        /// </summary>
         private List<Chapter> sourceChaptersList;
 
-        #endregion
-
-        #region Constructors and Destructors
-
-        /// <summary>
-        /// Initializes a new instance of the <see cref="ChaptersViewModel"/> class.
-        /// </summary>
-        /// <param name="windowManager">
-        /// The window manager.
-        /// </param>
-        /// <param name="userSettingService">
-        /// The user Setting Service.
-        /// </param>
-        /// <param name="errorService">
-        /// The Error Service 
-        /// </param>
         public ChaptersViewModel(IWindowManager windowManager, IUserSettingService userSettingService, IErrorService errorService)
         {
             this.Task = new EncodeTask();
             this.errorService = errorService;
         }
 
-        #endregion
+        public event EventHandler<TabStatusEventArgs> TabStatusChanged;
 
         #region Public Properties
+
         /// <summary>
         /// Gets or sets Task.
         /// </summary>
@@ -91,6 +65,21 @@ namespace HandBrakeWPF.ViewModels
             {
                 this.Task.IncludeChapterMarkers = value;
                 this.NotifyOfPropertyChange(() => this.IncludeChapterMarkers);
+                this.OnTabStatusChanged(null);
+            }
+        }
+
+        public ObservableCollection<ChapterMarker> Chapters
+        {
+            get
+            {
+                return this.Task.ChapterNames;
+            }
+
+            set
+            {
+                this.Task.ChapterNames = value;
+                this.NotifyOfPropertyChange(() => this.Chapters);
             }
         }
 
@@ -115,28 +104,27 @@ namespace HandBrakeWPF.ViewModels
         /// </exception>
         public void Export()
         {
-            string fileName = null;
-            using (var saveFileDialog = new SaveFileDialog()
-                                        {
-                                            Filter = "Csv File|*.csv",
-                                            DefaultExt = "csv",
-                                            CheckPathExists = true,
-                                            OverwritePrompt = true
-                                        })
-            {
-                var dialogResult = saveFileDialog.ShowDialog();
-                fileName = saveFileDialog.FileName;
+            var saveFileDialog = new SaveFileDialog()
+                                 {
+                                     Filter = "Csv File|*.csv",
+                                     DefaultExt = "csv",
+                                     CheckPathExists = true,
+                                     OverwritePrompt = true
+                                 };
 
-                // Exit early if the user cancelled or the filename is invalid
-                if (dialogResult != DialogResult.OK || string.IsNullOrWhiteSpace(fileName))
-                    return;
+            var dialogResult = saveFileDialog.ShowDialog();
+            string filename = saveFileDialog.FileName;
+
+            if (dialogResult == false || string.IsNullOrEmpty(filename))
+            {
+                return;
             }
 
             try
             {
-                using (var csv = new StreamWriter(fileName))
+                using (var csv = new StreamWriter(filename))
                 {
-                    foreach (ChapterMarker row in this.Task.ChapterNames)
+                    foreach (ChapterMarker row in this.Chapters)
                     {
                         csv.Write("{0},{1}{2}", row.ChapterNumber, CsvHelper.Escape(row.ChapterName), Environment.NewLine);
                     }
@@ -161,24 +149,30 @@ namespace HandBrakeWPF.ViewModels
         {
             string filename = null;
             string fileExtension = null;
-            using (var dialog = new OpenFileDialog()
-                    {
-                        Filter = string.Join("|", ChapterImporterCsv.FileFilter, ChapterImporterXml.FileFilter, ChapterImporterTxt.FileFilter),
-                        FilterIndex = 1,  // 1 based, the index value of the first filter entry is 1
-                        CheckFileExists = true
-                    })
+            var dialog = new OpenFileDialog()
             {
-                var dialogResult = dialog.ShowDialog();
-                filename = dialog.FileName;
+                Filter = string.Join(
+                                 "|",
+                                 "All Supported Formats (*.csv;*.tsv,*.xml,*.txt)|*.csv;*.tsv;*.xml;*.txt",
+                                 ChapterImporterCsv.FileFilter,
+                                 ChapterImporterXml.FileFilter,
+                                 ChapterImporterTxt.FileFilter),
+                FilterIndex = 1, // 1 based, the index value of the first filter entry is 1
+                CheckFileExists = true
+            };
 
-                // Exit if the user didn't press the OK button or the file name is invalid
-                if (dialogResult != DialogResult.OK || string.IsNullOrWhiteSpace(filename))
-                    return;
+            var dialogResult = dialog.ShowDialog();
+            filename = dialog.FileName;
 
-                // Retrieve the file extension after we've confirmed that the user selected something to open
-                fileExtension = Path.GetExtension(filename)?.ToLowerInvariant();
+            if (dialogResult == false || string.IsNullOrEmpty(filename))
+            {
+                return;
             }
 
+            // Retrieve the file extension after we've confirmed that the user selected something to open
+            fileExtension = Path.GetExtension(filename)?.ToLowerInvariant();
+
+
             var importedChapters = new Dictionary<int, Tuple<string, TimeSpan>>();
 
             // Execute the importer based on the file extension
@@ -199,26 +193,30 @@ namespace HandBrakeWPF.ViewModels
                         Resources.ChaptersViewModel_UnsupportedFileFormatWarning,
                         string.Format(Resources.ChaptersViewModel_UnsupportedFileFormatMsg, fileExtension));
             }
-            
+
             // Exit early if no chapter information was extracted
             if (importedChapters == null || importedChapters.Count <= 0)
                 return;
 
             // Validate the chaptermap against the current chapter list extracted from the source
+            bool hasTimestamps = importedChapters.Select(importedChapter => importedChapter.Value.Item2).Any(t => t != TimeSpan.Zero);
+
             string validationErrorMessage;
-            if (!this.ValidateImportedChapters(importedChapters, out validationErrorMessage))
+            if (!this.ValidateImportedChapters(importedChapters, out validationErrorMessage, hasTimestamps))
             {
                 if (!string.IsNullOrEmpty(validationErrorMessage))
+                {
                     throw new GeneralApplicationException(
                               Resources.ChaptersViewModel_ValidationFailedWarning,
                               validationErrorMessage);
+                }
 
                 // The user has cancelled the import, so exit
                 return;
             }
 
-            // Now iterate over each chatper we have, and set it's name
-            foreach (ChapterMarker item in this.Task.ChapterNames)
+            // Now iterate over each chapter we have, and set it's name
+            foreach (ChapterMarker item in this.Chapters)
             {
                 // If we don't have a chapter name for this chapter then 
                 // fallback to the value that is already set for the chapter
@@ -229,60 +227,11 @@ namespace HandBrakeWPF.ViewModels
                 if (importedChapters.TryGetValue(item.ChapterNumber, out chapterInfo))
                     chapterName = chapterInfo.Item1;
 
-                // Assign the chapter name unless the name is not set or only whitespace charaters
+                // Assign the chapter name unless the name is not set or only whitespace characters
                 item.ChapterName = !string.IsNullOrWhiteSpace(chapterName) ? chapterName : string.Empty;
             }
         }
 
-        /// <summary>
-        /// Validates any imported chapter information against the currently detected chapter information in the 
-        /// source media. If validation fails then an error message is returned via the out parameter <see cref="validationErrorMessage"/>
-        /// </summary>
-        /// <param name="importedChapters">The list of imported chapter information</param>
-        /// <param name="validationErrorMessage">In case of a validation error this variable will hold 
-        ///                                      a detailed message that can be presented to the user</param>
-        /// <returns>True if there are no errors with imported chapters, false otherwise</returns>
-        private bool ValidateImportedChapters(Dictionary<int, Tuple<string, TimeSpan>> importedChapters, out string validationErrorMessage)
-        {
-            validationErrorMessage = null;
-
-            // If the number of chapters don't match, prompt for confirmation
-            if (importedChapters.Count != this.Task.ChapterNames.Count)
-            {
-                if (MessageBoxResult.Yes !=
-                    this.errorService.ShowMessageBox(
-                        string.Format(Resources.ChaptersViewModel_ValidateImportedChapters_ChapterCountMismatchMsg, this.Task.ChapterNames.Count, importedChapters.Count),
-                        Resources.ChaptersViewModel_ValidateImportedChapters_ChapterCountMismatchWarning,
-                        MessageBoxButton.YesNo,
-                        MessageBoxImage.Question))
-                {
-                    return false;
-                }
-            }
-
-            // If the average discrepancy in timings between chapters is either:
-            //   a) more than 15 sec for more than 2 chapters
-            //      (I chose 15sec based on empirical evidence from testing a few DVDs and comparing to chapter-marker files I downloaded)
-            //      => This check will not be performed for the first and last chapter as they're very likely to differ significantly due to language and region
-            //         differences (e.g. longer title sequences and different distributor credits)
-            var diffs = importedChapters.Zip(this.Task.ChapterNames, (import, source) => source.Duration - import.Value.Item2);
-            if (diffs.Count(diff => Math.Abs(diff.TotalSeconds) > 15) > 2)
-            {
-                if (MessageBoxResult.Yes !=
-                    this.errorService.ShowMessageBox(
-                        Resources.ChaptersViewModel_ValidateImportedChapters_ChapterDurationMismatchMsg,
-                        Resources.ChaptersViewModel_ValidateImportedChapters_ChapterDurationMismatchWarning,
-                        MessageBoxButton.YesNo,
-                        MessageBoxImage.Question))
-                {
-                    return false;
-                }
-            }
-
-            // All is well, we should import chapters
-            return true;
-        }
-
         /// <summary>
         /// Setup this window for a new source
         /// </summary>
@@ -301,7 +250,6 @@ namespace HandBrakeWPF.ViewModels
         public void SetSource(Source source, Title title, Preset preset, EncodeTask task)
         {
             this.Task = task;
-            this.NotifyOfPropertyChange(() => this.Task);
 
             if (preset != null)
             {
@@ -324,8 +272,8 @@ namespace HandBrakeWPF.ViewModels
         public void SetPreset(Preset preset, EncodeTask task)
         {
             this.Task = task;
-            this.Task.IncludeChapterMarkers = preset.Task.IncludeChapterMarkers;
-            this.NotifyOfPropertyChange(() => this.Task);
+            this.IncludeChapterMarkers = preset.Task.IncludeChapterMarkers;
+            this.NotifyOfPropertyChange(() => this.Chapters);
         }
 
         /// <summary>
@@ -338,8 +286,18 @@ namespace HandBrakeWPF.ViewModels
         {
             this.Task = task;
 
-            this.NotifyOfPropertyChange(() => this.Task.IncludeChapterMarkers);
-            this.NotifyOfPropertyChange(() => this.Task.ChapterNames);
+            this.NotifyOfPropertyChange(() => this.IncludeChapterMarkers);
+            this.NotifyOfPropertyChange(() => this.Chapters);
+        }
+
+        public bool MatchesPreset(Preset preset)
+        {
+            if (preset.Task.IncludeChapterMarkers != this.IncludeChapterMarkers)
+            {
+                return false;
+            }
+
+            return true;
         }
 
         /// <summary>
@@ -363,21 +321,93 @@ namespace HandBrakeWPF.ViewModels
         {
             // Cache the chapters in this screen
             this.SourceChapterList = new ObservableCollection<Chapter>(sourceChapters);
-            this.Task.ChapterNames.Clear();
+            this.Chapters.Clear();
 
             // Then Add new Chapter Markers.
             int counter = 1;
 
             foreach (Chapter chapter in this.SourceChapterList)
             {
-                string chapterName = string.IsNullOrEmpty(chapter.ChapterName) ? string.Format("Chapter {0}", counter) : chapter.ChapterName;
+                string chapterName = string.IsNullOrEmpty(chapter.ChapterName) ? string.Format(Resources.ChapterViewModel_Chapter, counter) : chapter.ChapterName;
                 var marker = new ChapterMarker(chapter.ChapterNumber, chapterName, chapter.Duration);
-                this.Task.ChapterNames.Add(marker);
+                this.Chapters.Add(marker);
 
                 counter += 1;
             }
         }
 
         #endregion
+
+        #region Private Methods
+
+        protected virtual void OnTabStatusChanged(TabStatusEventArgs e)
+        {
+            this.TabStatusChanged?.Invoke(this, e);
+        }
+
+        /// <summary>
+        /// Validates any imported chapter information against the currently detected chapter information in the 
+        /// source media. If validation fails then an error message is returned via the out parameter <see cref="validationErrorMessage"/>
+        /// </summary>
+        /// <param name="importedChapters">The list of imported chapter information</param>
+        /// <param name="validationErrorMessage">In case of a validation error this variable will hold 
+        ///                                      a detailed message that can be presented to the user</param>
+        /// <returns>True if there are no errors with imported chapters, false otherwise</returns>
+        private bool ValidateImportedChapters(Dictionary<int, Tuple<string, TimeSpan>> importedChapters, out string validationErrorMessage, bool hasTimestamps)
+        {
+            validationErrorMessage = null;
+
+            // If the number of chapters don't match, prompt for confirmation
+            if (importedChapters.Count != this.Chapters.Count)
+            {
+                if (this.errorService.ShowMessageBox(
+                        string.Format(Resources.ChaptersViewModel_ValidateImportedChapters_ChapterCountMismatchMsg, this.Chapters.Count, importedChapters.Count),
+                        Resources.ChaptersViewModel_ValidateImportedChapters_ChapterCountMismatchWarning,
+                        MessageBoxButton.YesNo,
+                        MessageBoxImage.Question) !=
+                    MessageBoxResult.Yes)
+                {
+                    return false;
+                }
+            }
+
+            // If the average discrepancy in timings between chapters is either:
+            //   a) more than 15 sec for more than 2 chapters
+            //      (I chose 15sec based on empirical evidence from testing a few DVDs and comparing to chapter-marker files I downloaded)
+            //      => This check will not be performed for the first and last chapter as they're very likely to differ significantly due to language and region
+            //         differences (e.g. longer title sequences and different distributor credits)
+            if (hasTimestamps)
+            {
+                List<TimeSpan> diffs = new List<TimeSpan>();
+                foreach (KeyValuePair<int, Tuple<string, TimeSpan>> import in importedChapters)
+                {
+                    ChapterMarker sourceMarker = this.Chapters[import.Key - 1];
+                    TimeSpan source = sourceMarker.Duration;
+
+                    TimeSpan diff = source - import.Value.Item2;
+                    diffs.Add(diff);
+
+                }
+
+
+               // var diffs = importedChapters.Zip(this.Chapters, (import, source) => source.Duration - import.Value.Item2);
+                if (diffs.Count(diff => Math.Abs(diff.TotalSeconds) > 15) > 2)
+                {
+                    if (this.errorService.ShowMessageBox(
+                            Resources.ChaptersViewModel_ValidateImportedChapters_ChapterDurationMismatchMsg,
+                            Resources.ChaptersViewModel_ValidateImportedChapters_ChapterDurationMismatchWarning,
+                            MessageBoxButton.YesNo,
+                            MessageBoxImage.Question) != MessageBoxResult.Yes)
+                    {
+                        return false;
+                    }
+                }
+            }
+
+            // All is well, we should import chapters
+            return true;
+        }
+
+        #endregion
     }
 }
\ No newline at end of file
